@@ -11,7 +11,8 @@ function(x, n_cat, stack100,
          pad_y_min, pad_y_max,
          legend_title, legend_position, legend_labels,
          legend_horiz, legend_size, legend_abbrev, legend_adj,
-         out_size, quiet, width, height, pdf_file, ...)  {
+         out_size, do_plot, quiet, width, height, pdf_file, ...)  {
+
   sug.keep <- getOption("suggest")
   options(suggest = FALSE)
 
@@ -75,7 +76,7 @@ function(x, n_cat, stack100,
           legend_title, legend_position, legend_labels,
           legend_horiz, legend_size, legend_abbrev, legend_adj,
           add=NULL, x1=NULL, x2=NULL, y1=NULL, y2=NULL, out_size,
-          digits_d=3, quiet, ...)
+          digits_d=3, do_plot, quiet, ...)
 
         if (!is.null(pdf_file)) {
           dev.off()
